from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.post_evaluations_v3_run_response_200_status import PostEvaluationsV3RunResponse200Status
from ..types import UNSET, Unset

T = TypeVar("T", bound="PostEvaluationsV3RunResponse200")


@_attrs_define
class PostEvaluationsV3RunResponse200:
    """
    Attributes:
        run_id (str): Unique identifier for this run
        status (PostEvaluationsV3RunResponse200Status): Initial status of the run
        total (int): Total number of cells to execute
        run_url (str | Unset): URL to view the run in LangWatch
    """

    run_id: str
    status: PostEvaluationsV3RunResponse200Status
    total: int
    run_url: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        run_id = self.run_id

        status = self.status.value

        total = self.total

        run_url = self.run_url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "runId": run_id,
                "status": status,
                "total": total,
            }
        )
        if run_url is not UNSET:
            field_dict["runUrl"] = run_url

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        run_id = d.pop("runId")

        status = PostEvaluationsV3RunResponse200Status(d.pop("status"))

        total = d.pop("total")

        run_url = d.pop("runUrl", UNSET)

        post_evaluations_v3_run_response_200 = cls(
            run_id=run_id,
            status=status,
            total=total,
            run_url=run_url,
        )

        post_evaluations_v3_run_response_200.additional_properties = d
        return post_evaluations_v3_run_response_200

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import List, Any, Dict, Union, Optional, cast, TYPE_CHECKING

from pydantic import BaseModel, ConfigDict, Field, PrivateAttr, computed_field

if TYPE_CHECKING:
    from openai.types.chat import ChatCompletionMessageParam

from liquid import Environment, StrictUndefined, Undefined
from liquid.exceptions import UndefinedError
from .decorators.prompt_tracing import prompt_tracing
from .types import PromptData, Message


class PromptCompilationError(Exception):
    """Error class for template compilation issues"""

    def __init__(
        self, message: str, template: str, original_error: Optional[Exception] = None
    ):
        super().__init__(message)
        self.template = template
        self.original_error = original_error
        self.name = "PromptCompilationError"


# Type for template variables - supporting common data types
TemplateVariables = Dict[
    str, Union[str, int, float, bool, Dict[str, Any], List[Any], None]
]


class Prompt(PromptData):
    """
    A class representing a prompt configuration that can be used with OpenAI's API.
    Handles formatting messages with variables using Liquid templating.
    """

    _raw_data: Optional[PromptData] = PrivateAttr(default=None)

    def __init__(
        self,
        data: Union["PromptData", Dict[str, Any], None] = None,
        **kwargs: Any,
    ):
        if data is not None:
            if isinstance(data, BaseModel):
                init_kwargs = data.model_dump()
            elif isinstance(data, dict):
                init_kwargs = dict(data)
            else:
                init_kwargs = {}
            init_kwargs.update(kwargs)
            super().__init__(**init_kwargs)
            if isinstance(data, PromptData):
                self._raw_data = data.model_copy()
            else:
                self._raw_data = PromptData(**init_kwargs)
        else:
            super().__init__(**kwargs)
            self._raw_data = PromptData(**kwargs)

        # Set prompt default only if not provided (like TypeScript)
        if self.prompt is None:
            self.prompt = self._extract_system_prompt()

    @property
    def raw(self) -> PromptData:
        """Get the raw prompt data"""
        if self._raw_data is not None:
            return self._raw_data
        return PromptData(**self.model_dump())

    def _extract_system_prompt(self) -> str:
        """Extract system prompt from messages, like TypeScript version."""
        if self.messages:
            for message in self.messages:
                if message.role == "system":
                    return message.content
        return ""

    def _compile(self, variables: TemplateVariables, strict: bool) -> "CompiledPrompt":
        """
        Internal method to compile the prompt template with provided variables.
        """
        try:
            # Create environment based on strict mode
            env = Environment(undefined=StrictUndefined if strict else Undefined)

            # Compile main prompt
            compiled_prompt = ""
            if self.prompt:
                template = env.from_string(self.prompt)
                compiled_prompt = template.render(**variables)

            # Compile messages
            compiled_messages: List[Message] = []
            if self.messages:
                for message in self.messages:
                    template = env.from_string(message.content)
                    compiled_content = template.render(**variables)
                    compiled_message = Message(
                        role=message.role,
                        content=compiled_content,
                    )
                    compiled_messages.append(compiled_message)

            # Return simplified CompiledPrompt with variables preserved
            return CompiledPrompt(
                original=self,
                prompt=compiled_prompt,
                compiled_messages=compiled_messages,
                variables=variables.copy(),
            )

        except UndefinedError as error:
            template_str = self.prompt or str(self.messages)
            raise PromptCompilationError(
                f"Failed to compile prompt template: {str(error)}", template_str, error
            )
        except Exception as error:
            template_str = self.prompt or str(self.messages)
            raise PromptCompilationError(
                f"Failed to compile prompt template: {str(error)}", template_str, error
            )

    @prompt_tracing.compile
    def compile(
        self, variables: Optional[TemplateVariables] = None, **kwargs: Any
    ) -> "CompiledPrompt":
        """
        Compile the prompt template with provided variables (lenient - missing variables become empty).

        Args:
            variables: Dictionary containing variable values for template compilation
            **kwargs: Alternative way to pass variables as keyword arguments

        Returns:
            CompiledPrompt instance with compiled content
        """
        if variables is None:
            variables = {}

        # Merge explicit dict with kwargs, kwargs take precedence for conflicts
        merged_variables = {**variables, **kwargs}
        return self._compile(merged_variables, strict=False)

    @prompt_tracing.compile_strict
    def compile_strict(
        self, variables: Optional[TemplateVariables] = None, **kwargs: Any
    ) -> "CompiledPrompt":
        """
        Compile with validation - throws error if required variables are missing.

        Args:
            variables: Template variables

        Returns:
            CompiledPrompt instance with compiled content

        Raises:
            PromptCompilationError: If required variables are missing or compilation fails
        """
        if variables is None:
            variables = {}

        # Merge explicit dict with kwargs, kwargs take precedence for conflicts
        merged_variables = {**variables, **kwargs}
        return self._compile(merged_variables, strict=True)


class CompiledPrompt(BaseModel):
    """
    Represents a compiled prompt with compiled content and original variables
    """

    model_config = ConfigDict(arbitrary_types_allowed=True)

    original: Prompt = Field(repr=False)
    prompt: str
    compiled_messages: List[Message] = Field(repr=False)
    variables: TemplateVariables

    @computed_field  # type: ignore[prop-decorator]
    @property
    def messages(self) -> List[Dict[str, str]]:
        """
        Returns the compiled messages as a list of dicts compatible with OpenAI's API.
        """
        return [msg.model_dump() for msg in self.compiled_messages]

    def __getattr__(self, name: str) -> Any:
        """Delegate unknown attributes to original prompt for backward compatibility"""
        return getattr(self.original, name)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.get_api_evaluators_by_id_or_slug_response_200_config_type_0 import (
        GetApiEvaluatorsByIdOrSlugResponse200ConfigType0,
    )
    from ..models.get_api_evaluators_by_id_or_slug_response_200_fields_item import (
        GetApiEvaluatorsByIdOrSlugResponse200FieldsItem,
    )
    from ..models.get_api_evaluators_by_id_or_slug_response_200_output_fields_item import (
        GetApiEvaluatorsByIdOrSlugResponse200OutputFieldsItem,
    )


T = TypeVar("T", bound="GetApiEvaluatorsByIdOrSlugResponse200")


@_attrs_define
class GetApiEvaluatorsByIdOrSlugResponse200:
    """
    Attributes:
        id (str):
        project_id (str):
        name (str):
        slug (None | str):
        type_ (str):
        config (GetApiEvaluatorsByIdOrSlugResponse200ConfigType0 | None):
        workflow_id (None | str):
        copied_from_evaluator_id (None | str):
        created_at (float | str):
        updated_at (float | str):
        fields (list[GetApiEvaluatorsByIdOrSlugResponse200FieldsItem]):
        output_fields (list[GetApiEvaluatorsByIdOrSlugResponse200OutputFieldsItem]):
        workflow_name (str | Unset):
        workflow_icon (str | Unset):
    """

    id: str
    project_id: str
    name: str
    slug: None | str
    type_: str
    config: GetApiEvaluatorsByIdOrSlugResponse200ConfigType0 | None
    workflow_id: None | str
    copied_from_evaluator_id: None | str
    created_at: float | str
    updated_at: float | str
    fields: list[GetApiEvaluatorsByIdOrSlugResponse200FieldsItem]
    output_fields: list[GetApiEvaluatorsByIdOrSlugResponse200OutputFieldsItem]
    workflow_name: str | Unset = UNSET
    workflow_icon: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.get_api_evaluators_by_id_or_slug_response_200_config_type_0 import (
            GetApiEvaluatorsByIdOrSlugResponse200ConfigType0,
        )

        id = self.id

        project_id = self.project_id

        name = self.name

        slug: None | str
        slug = self.slug

        type_ = self.type_

        config: dict[str, Any] | None
        if isinstance(self.config, GetApiEvaluatorsByIdOrSlugResponse200ConfigType0):
            config = self.config.to_dict()
        else:
            config = self.config

        workflow_id: None | str
        workflow_id = self.workflow_id

        copied_from_evaluator_id: None | str
        copied_from_evaluator_id = self.copied_from_evaluator_id

        created_at: float | str
        created_at = self.created_at

        updated_at: float | str
        updated_at = self.updated_at

        fields = []
        for fields_item_data in self.fields:
            fields_item = fields_item_data.to_dict()
            fields.append(fields_item)

        output_fields = []
        for output_fields_item_data in self.output_fields:
            output_fields_item = output_fields_item_data.to_dict()
            output_fields.append(output_fields_item)

        workflow_name = self.workflow_name

        workflow_icon = self.workflow_icon

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "projectId": project_id,
                "name": name,
                "slug": slug,
                "type": type_,
                "config": config,
                "workflowId": workflow_id,
                "copiedFromEvaluatorId": copied_from_evaluator_id,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "fields": fields,
                "outputFields": output_fields,
            }
        )
        if workflow_name is not UNSET:
            field_dict["workflowName"] = workflow_name
        if workflow_icon is not UNSET:
            field_dict["workflowIcon"] = workflow_icon

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_api_evaluators_by_id_or_slug_response_200_config_type_0 import (
            GetApiEvaluatorsByIdOrSlugResponse200ConfigType0,
        )
        from ..models.get_api_evaluators_by_id_or_slug_response_200_fields_item import (
            GetApiEvaluatorsByIdOrSlugResponse200FieldsItem,
        )
        from ..models.get_api_evaluators_by_id_or_slug_response_200_output_fields_item import (
            GetApiEvaluatorsByIdOrSlugResponse200OutputFieldsItem,
        )

        d = dict(src_dict)
        id = d.pop("id")

        project_id = d.pop("projectId")

        name = d.pop("name")

        def _parse_slug(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        slug = _parse_slug(d.pop("slug"))

        type_ = d.pop("type")

        def _parse_config(data: object) -> GetApiEvaluatorsByIdOrSlugResponse200ConfigType0 | None:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                config_type_0 = GetApiEvaluatorsByIdOrSlugResponse200ConfigType0.from_dict(data)

                return config_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(GetApiEvaluatorsByIdOrSlugResponse200ConfigType0 | None, data)

        config = _parse_config(d.pop("config"))

        def _parse_workflow_id(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        workflow_id = _parse_workflow_id(d.pop("workflowId"))

        def _parse_copied_from_evaluator_id(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        copied_from_evaluator_id = _parse_copied_from_evaluator_id(d.pop("copiedFromEvaluatorId"))

        def _parse_created_at(data: object) -> float | str:
            return cast(float | str, data)

        created_at = _parse_created_at(d.pop("createdAt"))

        def _parse_updated_at(data: object) -> float | str:
            return cast(float | str, data)

        updated_at = _parse_updated_at(d.pop("updatedAt"))

        fields = []
        _fields = d.pop("fields")
        for fields_item_data in _fields:
            fields_item = GetApiEvaluatorsByIdOrSlugResponse200FieldsItem.from_dict(fields_item_data)

            fields.append(fields_item)

        output_fields = []
        _output_fields = d.pop("outputFields")
        for output_fields_item_data in _output_fields:
            output_fields_item = GetApiEvaluatorsByIdOrSlugResponse200OutputFieldsItem.from_dict(
                output_fields_item_data
            )

            output_fields.append(output_fields_item)

        workflow_name = d.pop("workflowName", UNSET)

        workflow_icon = d.pop("workflowIcon", UNSET)

        get_api_evaluators_by_id_or_slug_response_200 = cls(
            id=id,
            project_id=project_id,
            name=name,
            slug=slug,
            type_=type_,
            config=config,
            workflow_id=workflow_id,
            copied_from_evaluator_id=copied_from_evaluator_id,
            created_at=created_at,
            updated_at=updated_at,
            fields=fields,
            output_fields=output_fields,
            workflow_name=workflow_name,
            workflow_icon=workflow_icon,
        )

        get_api_evaluators_by_id_or_slug_response_200.additional_properties = d
        return get_api_evaluators_by_id_or_slug_response_200

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PostApiAnalyticsTimeseriesBodySeriesItemMetric(str, Enum):
    EVALUATIONS_EVALUATION_PASS_RATE = "evaluations.evaluation_pass_rate"
    EVALUATIONS_EVALUATION_RUNS = "evaluations.evaluation_runs"
    EVALUATIONS_EVALUATION_SCORE = "evaluations.evaluation_score"
    EVENTS_EVENT_DETAILS = "events.event_details"
    EVENTS_EVENT_SCORE = "events.event_score"
    EVENTS_EVENT_TYPE = "events.event_type"
    METADATA_SPAN_TYPE = "metadata.span_type"
    METADATA_THREAD_ID = "metadata.thread_id"
    METADATA_TRACE_ID = "metadata.trace_id"
    METADATA_USER_ID = "metadata.user_id"
    PERFORMANCE_COMPLETION_TIME = "performance.completion_time"
    PERFORMANCE_COMPLETION_TOKENS = "performance.completion_tokens"
    PERFORMANCE_FIRST_TOKEN = "performance.first_token"
    PERFORMANCE_PROMPT_TOKENS = "performance.prompt_tokens"
    PERFORMANCE_TOKENS_PER_SECOND = "performance.tokens_per_second"
    PERFORMANCE_TOTAL_COST = "performance.total_cost"
    PERFORMANCE_TOTAL_TOKENS = "performance.total_tokens"
    SENTIMENT_INPUT_SENTIMENT = "sentiment.input_sentiment"
    SENTIMENT_THUMBS_UP_DOWN = "sentiment.thumbs_up_down"
    THREADS_AVERAGE_DURATION_PER_THREAD = "threads.average_duration_per_thread"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class PostApiAnalyticsTimeseriesBodySeriesItemAggregation(str, Enum):
    AVG = "avg"
    CARDINALITY = "cardinality"
    MAX = "max"
    MEDIAN = "median"
    MIN = "min"
    P90 = "p90"
    P95 = "p95"
    P99 = "p99"
    SUM = "sum"
    TERMS = "terms"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.post_api_analytics_timeseries_body_filters_annotations_has_annotation_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_annotations_has_annotation_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_guardrails_only_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_label_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_label_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_passed_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_passed_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_score_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_score_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_state_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_evaluations_state_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_events_event_details_key_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_events_event_details_key_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_events_event_type_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_events_event_type_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_events_metrics_key_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_events_metrics_key_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_events_metrics_value_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_events_metrics_value_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_customer_id_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_customer_id_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_key_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_key_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_labels_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_labels_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_prompt_ids_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_prompt_ids_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_thread_id_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_thread_id_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_user_id_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_user_id_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_value_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_metadata_value_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_sentiment_input_sentiment_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_sentiment_input_sentiment_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_spans_model_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersSpansModelType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_spans_model_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersSpansModelType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_spans_type_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_spans_type_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_topics_subtopics_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_topics_subtopics_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_topics_topics_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_topics_topics_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType2,
    )
    from ..models.post_api_analytics_timeseries_body_filters_traces_error_type_1 import (
        PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType1,
    )
    from ..models.post_api_analytics_timeseries_body_filters_traces_error_type_2 import (
        PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType2,
    )


T = TypeVar("T", bound="PostApiAnalyticsTimeseriesBodyFilters")


@_attrs_define
class PostApiAnalyticsTimeseriesBodyFilters:
    """
    Attributes:
        topics_topics (list[str] | PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType1 |
            PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType2 | Unset):
        topics_subtopics (list[str] | PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType1 |
            PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType2 | Unset):
        metadata_user_id (list[str] | PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType1 |
            PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType2 | Unset):
        metadata_thread_id (list[str] | PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType1 |
            PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType2 | Unset):
        metadata_customer_id (list[str] | PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType1 |
            PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType2 | Unset):
        metadata_labels (list[str] | PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType1 |
            PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType2 | Unset):
        metadata_key (list[str] | PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType1 |
            PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType2 | Unset):
        metadata_value (list[str] | PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType1 |
            PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType2 | Unset):
        metadata_prompt_ids (list[str] | PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType1 |
            PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType2 | Unset):
        traces_error (list[str] | PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType1 |
            PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType2 | Unset):
        spans_type (list[str] | PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType1 |
            PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType2 | Unset):
        spans_model (list[str] | PostApiAnalyticsTimeseriesBodyFiltersSpansModelType1 |
            PostApiAnalyticsTimeseriesBodyFiltersSpansModelType2 | Unset):
        evaluations_evaluator_id (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType2 | Unset):
        evaluations_evaluator_id_guardrails_only (list[str] |
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2 | Unset):
        evaluations_passed (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType2 | Unset):
        evaluations_score (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType2 | Unset):
        evaluations_state (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType2 | Unset):
        evaluations_label (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType2 | Unset):
        events_event_type (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType2 | Unset):
        events_metrics_key (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType2 | Unset):
        events_metrics_value (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType2 | Unset):
        events_event_details_key (list[str] | PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType1 |
            PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType2 | Unset):
        annotations_has_annotation (list[str] | PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType1 |
            PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType2 | Unset):
        sentiment_input_sentiment (list[str] | PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType1 |
            PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType2 | Unset):
    """

    topics_topics: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType1
        | PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType2
        | Unset
    ) = UNSET
    topics_subtopics: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType1
        | PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType2
        | Unset
    ) = UNSET
    metadata_user_id: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType1
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType2
        | Unset
    ) = UNSET
    metadata_thread_id: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType1
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType2
        | Unset
    ) = UNSET
    metadata_customer_id: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType1
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType2
        | Unset
    ) = UNSET
    metadata_labels: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType1
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType2
        | Unset
    ) = UNSET
    metadata_key: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType1
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType2
        | Unset
    ) = UNSET
    metadata_value: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType1
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType2
        | Unset
    ) = UNSET
    metadata_prompt_ids: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType1
        | PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType2
        | Unset
    ) = UNSET
    traces_error: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType1
        | PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType2
        | Unset
    ) = UNSET
    spans_type: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType1
        | PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType2
        | Unset
    ) = UNSET
    spans_model: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersSpansModelType1
        | PostApiAnalyticsTimeseriesBodyFiltersSpansModelType2
        | Unset
    ) = UNSET
    evaluations_evaluator_id: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType1
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType2
        | Unset
    ) = UNSET
    evaluations_evaluator_id_guardrails_only: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2
        | Unset
    ) = UNSET
    evaluations_passed: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType1
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType2
        | Unset
    ) = UNSET
    evaluations_score: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType1
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType2
        | Unset
    ) = UNSET
    evaluations_state: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType1
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType2
        | Unset
    ) = UNSET
    evaluations_label: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType1
        | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType2
        | Unset
    ) = UNSET
    events_event_type: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType1
        | PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType2
        | Unset
    ) = UNSET
    events_metrics_key: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType1
        | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType2
        | Unset
    ) = UNSET
    events_metrics_value: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType1
        | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType2
        | Unset
    ) = UNSET
    events_event_details_key: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType1
        | PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType2
        | Unset
    ) = UNSET
    annotations_has_annotation: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType1
        | PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType2
        | Unset
    ) = UNSET
    sentiment_input_sentiment: (
        list[str]
        | PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType1
        | PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType2
        | Unset
    ) = UNSET

    def to_dict(self) -> dict[str, Any]:
        from ..models.post_api_analytics_timeseries_body_filters_annotations_has_annotation_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_label_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_passed_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_score_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_state_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_event_details_key_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_event_type_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_metrics_key_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_metrics_value_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_customer_id_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_key_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_labels_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_prompt_ids_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_thread_id_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_user_id_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_value_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_sentiment_input_sentiment_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_spans_model_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersSpansModelType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_spans_type_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_topics_subtopics_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_topics_topics_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_traces_error_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType1,
        )

        topics_topics: dict[str, Any] | list[str] | Unset
        if isinstance(self.topics_topics, Unset):
            topics_topics = UNSET
        elif isinstance(self.topics_topics, list):
            topics_topics = self.topics_topics

        elif isinstance(self.topics_topics, PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType1):
            topics_topics = self.topics_topics.to_dict()
        else:
            topics_topics = self.topics_topics.to_dict()

        topics_subtopics: dict[str, Any] | list[str] | Unset
        if isinstance(self.topics_subtopics, Unset):
            topics_subtopics = UNSET
        elif isinstance(self.topics_subtopics, list):
            topics_subtopics = self.topics_subtopics

        elif isinstance(self.topics_subtopics, PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType1):
            topics_subtopics = self.topics_subtopics.to_dict()
        else:
            topics_subtopics = self.topics_subtopics.to_dict()

        metadata_user_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_user_id, Unset):
            metadata_user_id = UNSET
        elif isinstance(self.metadata_user_id, list):
            metadata_user_id = self.metadata_user_id

        elif isinstance(self.metadata_user_id, PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType1):
            metadata_user_id = self.metadata_user_id.to_dict()
        else:
            metadata_user_id = self.metadata_user_id.to_dict()

        metadata_thread_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_thread_id, Unset):
            metadata_thread_id = UNSET
        elif isinstance(self.metadata_thread_id, list):
            metadata_thread_id = self.metadata_thread_id

        elif isinstance(self.metadata_thread_id, PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType1):
            metadata_thread_id = self.metadata_thread_id.to_dict()
        else:
            metadata_thread_id = self.metadata_thread_id.to_dict()

        metadata_customer_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_customer_id, Unset):
            metadata_customer_id = UNSET
        elif isinstance(self.metadata_customer_id, list):
            metadata_customer_id = self.metadata_customer_id

        elif isinstance(self.metadata_customer_id, PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType1):
            metadata_customer_id = self.metadata_customer_id.to_dict()
        else:
            metadata_customer_id = self.metadata_customer_id.to_dict()

        metadata_labels: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_labels, Unset):
            metadata_labels = UNSET
        elif isinstance(self.metadata_labels, list):
            metadata_labels = self.metadata_labels

        elif isinstance(self.metadata_labels, PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType1):
            metadata_labels = self.metadata_labels.to_dict()
        else:
            metadata_labels = self.metadata_labels.to_dict()

        metadata_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_key, Unset):
            metadata_key = UNSET
        elif isinstance(self.metadata_key, list):
            metadata_key = self.metadata_key

        elif isinstance(self.metadata_key, PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType1):
            metadata_key = self.metadata_key.to_dict()
        else:
            metadata_key = self.metadata_key.to_dict()

        metadata_value: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_value, Unset):
            metadata_value = UNSET
        elif isinstance(self.metadata_value, list):
            metadata_value = self.metadata_value

        elif isinstance(self.metadata_value, PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType1):
            metadata_value = self.metadata_value.to_dict()
        else:
            metadata_value = self.metadata_value.to_dict()

        metadata_prompt_ids: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_prompt_ids, Unset):
            metadata_prompt_ids = UNSET
        elif isinstance(self.metadata_prompt_ids, list):
            metadata_prompt_ids = self.metadata_prompt_ids

        elif isinstance(self.metadata_prompt_ids, PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType1):
            metadata_prompt_ids = self.metadata_prompt_ids.to_dict()
        else:
            metadata_prompt_ids = self.metadata_prompt_ids.to_dict()

        traces_error: dict[str, Any] | list[str] | Unset
        if isinstance(self.traces_error, Unset):
            traces_error = UNSET
        elif isinstance(self.traces_error, list):
            traces_error = self.traces_error

        elif isinstance(self.traces_error, PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType1):
            traces_error = self.traces_error.to_dict()
        else:
            traces_error = self.traces_error.to_dict()

        spans_type: dict[str, Any] | list[str] | Unset
        if isinstance(self.spans_type, Unset):
            spans_type = UNSET
        elif isinstance(self.spans_type, list):
            spans_type = self.spans_type

        elif isinstance(self.spans_type, PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType1):
            spans_type = self.spans_type.to_dict()
        else:
            spans_type = self.spans_type.to_dict()

        spans_model: dict[str, Any] | list[str] | Unset
        if isinstance(self.spans_model, Unset):
            spans_model = UNSET
        elif isinstance(self.spans_model, list):
            spans_model = self.spans_model

        elif isinstance(self.spans_model, PostApiAnalyticsTimeseriesBodyFiltersSpansModelType1):
            spans_model = self.spans_model.to_dict()
        else:
            spans_model = self.spans_model.to_dict()

        evaluations_evaluator_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_evaluator_id, Unset):
            evaluations_evaluator_id = UNSET
        elif isinstance(self.evaluations_evaluator_id, list):
            evaluations_evaluator_id = self.evaluations_evaluator_id

        elif isinstance(
            self.evaluations_evaluator_id, PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType1
        ):
            evaluations_evaluator_id = self.evaluations_evaluator_id.to_dict()
        else:
            evaluations_evaluator_id = self.evaluations_evaluator_id.to_dict()

        evaluations_evaluator_id_guardrails_only: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_evaluator_id_guardrails_only, Unset):
            evaluations_evaluator_id_guardrails_only = UNSET
        elif isinstance(self.evaluations_evaluator_id_guardrails_only, list):
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only

        elif isinstance(
            self.evaluations_evaluator_id_guardrails_only,
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        ):
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only.to_dict()
        else:
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only.to_dict()

        evaluations_passed: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_passed, Unset):
            evaluations_passed = UNSET
        elif isinstance(self.evaluations_passed, list):
            evaluations_passed = self.evaluations_passed

        elif isinstance(self.evaluations_passed, PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType1):
            evaluations_passed = self.evaluations_passed.to_dict()
        else:
            evaluations_passed = self.evaluations_passed.to_dict()

        evaluations_score: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_score, Unset):
            evaluations_score = UNSET
        elif isinstance(self.evaluations_score, list):
            evaluations_score = self.evaluations_score

        elif isinstance(self.evaluations_score, PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType1):
            evaluations_score = self.evaluations_score.to_dict()
        else:
            evaluations_score = self.evaluations_score.to_dict()

        evaluations_state: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_state, Unset):
            evaluations_state = UNSET
        elif isinstance(self.evaluations_state, list):
            evaluations_state = self.evaluations_state

        elif isinstance(self.evaluations_state, PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType1):
            evaluations_state = self.evaluations_state.to_dict()
        else:
            evaluations_state = self.evaluations_state.to_dict()

        evaluations_label: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_label, Unset):
            evaluations_label = UNSET
        elif isinstance(self.evaluations_label, list):
            evaluations_label = self.evaluations_label

        elif isinstance(self.evaluations_label, PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType1):
            evaluations_label = self.evaluations_label.to_dict()
        else:
            evaluations_label = self.evaluations_label.to_dict()

        events_event_type: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_event_type, Unset):
            events_event_type = UNSET
        elif isinstance(self.events_event_type, list):
            events_event_type = self.events_event_type

        elif isinstance(self.events_event_type, PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType1):
            events_event_type = self.events_event_type.to_dict()
        else:
            events_event_type = self.events_event_type.to_dict()

        events_metrics_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_metrics_key, Unset):
            events_metrics_key = UNSET
        elif isinstance(self.events_metrics_key, list):
            events_metrics_key = self.events_metrics_key

        elif isinstance(self.events_metrics_key, PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType1):
            events_metrics_key = self.events_metrics_key.to_dict()
        else:
            events_metrics_key = self.events_metrics_key.to_dict()

        events_metrics_value: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_metrics_value, Unset):
            events_metrics_value = UNSET
        elif isinstance(self.events_metrics_value, list):
            events_metrics_value = self.events_metrics_value

        elif isinstance(self.events_metrics_value, PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType1):
            events_metrics_value = self.events_metrics_value.to_dict()
        else:
            events_metrics_value = self.events_metrics_value.to_dict()

        events_event_details_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_event_details_key, Unset):
            events_event_details_key = UNSET
        elif isinstance(self.events_event_details_key, list):
            events_event_details_key = self.events_event_details_key

        elif isinstance(self.events_event_details_key, PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType1):
            events_event_details_key = self.events_event_details_key.to_dict()
        else:
            events_event_details_key = self.events_event_details_key.to_dict()

        annotations_has_annotation: dict[str, Any] | list[str] | Unset
        if isinstance(self.annotations_has_annotation, Unset):
            annotations_has_annotation = UNSET
        elif isinstance(self.annotations_has_annotation, list):
            annotations_has_annotation = self.annotations_has_annotation

        elif isinstance(
            self.annotations_has_annotation, PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType1
        ):
            annotations_has_annotation = self.annotations_has_annotation.to_dict()
        else:
            annotations_has_annotation = self.annotations_has_annotation.to_dict()

        sentiment_input_sentiment: dict[str, Any] | list[str] | Unset
        if isinstance(self.sentiment_input_sentiment, Unset):
            sentiment_input_sentiment = UNSET
        elif isinstance(self.sentiment_input_sentiment, list):
            sentiment_input_sentiment = self.sentiment_input_sentiment

        elif isinstance(
            self.sentiment_input_sentiment, PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType1
        ):
            sentiment_input_sentiment = self.sentiment_input_sentiment.to_dict()
        else:
            sentiment_input_sentiment = self.sentiment_input_sentiment.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if topics_topics is not UNSET:
            field_dict["topics.topics"] = topics_topics
        if topics_subtopics is not UNSET:
            field_dict["topics.subtopics"] = topics_subtopics
        if metadata_user_id is not UNSET:
            field_dict["metadata.user_id"] = metadata_user_id
        if metadata_thread_id is not UNSET:
            field_dict["metadata.thread_id"] = metadata_thread_id
        if metadata_customer_id is not UNSET:
            field_dict["metadata.customer_id"] = metadata_customer_id
        if metadata_labels is not UNSET:
            field_dict["metadata.labels"] = metadata_labels
        if metadata_key is not UNSET:
            field_dict["metadata.key"] = metadata_key
        if metadata_value is not UNSET:
            field_dict["metadata.value"] = metadata_value
        if metadata_prompt_ids is not UNSET:
            field_dict["metadata.prompt_ids"] = metadata_prompt_ids
        if traces_error is not UNSET:
            field_dict["traces.error"] = traces_error
        if spans_type is not UNSET:
            field_dict["spans.type"] = spans_type
        if spans_model is not UNSET:
            field_dict["spans.model"] = spans_model
        if evaluations_evaluator_id is not UNSET:
            field_dict["evaluations.evaluator_id"] = evaluations_evaluator_id
        if evaluations_evaluator_id_guardrails_only is not UNSET:
            field_dict["evaluations.evaluator_id.guardrails_only"] = evaluations_evaluator_id_guardrails_only
        if evaluations_passed is not UNSET:
            field_dict["evaluations.passed"] = evaluations_passed
        if evaluations_score is not UNSET:
            field_dict["evaluations.score"] = evaluations_score
        if evaluations_state is not UNSET:
            field_dict["evaluations.state"] = evaluations_state
        if evaluations_label is not UNSET:
            field_dict["evaluations.label"] = evaluations_label
        if events_event_type is not UNSET:
            field_dict["events.event_type"] = events_event_type
        if events_metrics_key is not UNSET:
            field_dict["events.metrics.key"] = events_metrics_key
        if events_metrics_value is not UNSET:
            field_dict["events.metrics.value"] = events_metrics_value
        if events_event_details_key is not UNSET:
            field_dict["events.event_details.key"] = events_event_details_key
        if annotations_has_annotation is not UNSET:
            field_dict["annotations.hasAnnotation"] = annotations_has_annotation
        if sentiment_input_sentiment is not UNSET:
            field_dict["sentiment.input_sentiment"] = sentiment_input_sentiment

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_analytics_timeseries_body_filters_annotations_has_annotation_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_annotations_has_annotation_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_guardrails_only_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_evaluator_id_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_label_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_label_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_passed_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_passed_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_score_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_score_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_state_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_evaluations_state_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_event_details_key_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_event_details_key_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_event_type_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_event_type_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_metrics_key_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_metrics_key_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_metrics_value_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_events_metrics_value_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_customer_id_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_customer_id_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_key_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_key_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_labels_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_labels_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_prompt_ids_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_prompt_ids_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_thread_id_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_thread_id_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_user_id_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_user_id_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_value_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_metadata_value_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_sentiment_input_sentiment_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_sentiment_input_sentiment_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_spans_model_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersSpansModelType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_spans_model_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersSpansModelType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_spans_type_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_spans_type_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_topics_subtopics_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_topics_subtopics_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_topics_topics_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_topics_topics_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType2,
        )
        from ..models.post_api_analytics_timeseries_body_filters_traces_error_type_1 import (
            PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType1,
        )
        from ..models.post_api_analytics_timeseries_body_filters_traces_error_type_2 import (
            PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType2,
        )

        d = dict(src_dict)

        def _parse_topics_topics(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType1
            | PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                topics_topics_type_0 = cast(list[str], data)

                return topics_topics_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                topics_topics_type_1 = PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType1.from_dict(data)

                return topics_topics_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            topics_topics_type_2 = PostApiAnalyticsTimeseriesBodyFiltersTopicsTopicsType2.from_dict(data)

            return topics_topics_type_2

        topics_topics = _parse_topics_topics(d.pop("topics.topics", UNSET))

        def _parse_topics_subtopics(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType1
            | PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                topics_subtopics_type_0 = cast(list[str], data)

                return topics_subtopics_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                topics_subtopics_type_1 = PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType1.from_dict(data)

                return topics_subtopics_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            topics_subtopics_type_2 = PostApiAnalyticsTimeseriesBodyFiltersTopicsSubtopicsType2.from_dict(data)

            return topics_subtopics_type_2

        topics_subtopics = _parse_topics_subtopics(d.pop("topics.subtopics", UNSET))

        def _parse_metadata_user_id(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType1
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_user_id_type_0 = cast(list[str], data)

                return metadata_user_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_user_id_type_1 = PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType1.from_dict(data)

                return metadata_user_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_user_id_type_2 = PostApiAnalyticsTimeseriesBodyFiltersMetadataUserIdType2.from_dict(data)

            return metadata_user_id_type_2

        metadata_user_id = _parse_metadata_user_id(d.pop("metadata.user_id", UNSET))

        def _parse_metadata_thread_id(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType1
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_thread_id_type_0 = cast(list[str], data)

                return metadata_thread_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_thread_id_type_1 = PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType1.from_dict(data)

                return metadata_thread_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_thread_id_type_2 = PostApiAnalyticsTimeseriesBodyFiltersMetadataThreadIdType2.from_dict(data)

            return metadata_thread_id_type_2

        metadata_thread_id = _parse_metadata_thread_id(d.pop("metadata.thread_id", UNSET))

        def _parse_metadata_customer_id(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType1
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_customer_id_type_0 = cast(list[str], data)

                return metadata_customer_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_customer_id_type_1 = PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType1.from_dict(
                    data
                )

                return metadata_customer_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_customer_id_type_2 = PostApiAnalyticsTimeseriesBodyFiltersMetadataCustomerIdType2.from_dict(data)

            return metadata_customer_id_type_2

        metadata_customer_id = _parse_metadata_customer_id(d.pop("metadata.customer_id", UNSET))

        def _parse_metadata_labels(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType1
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_labels_type_0 = cast(list[str], data)

                return metadata_labels_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_labels_type_1 = PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType1.from_dict(data)

                return metadata_labels_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_labels_type_2 = PostApiAnalyticsTimeseriesBodyFiltersMetadataLabelsType2.from_dict(data)

            return metadata_labels_type_2

        metadata_labels = _parse_metadata_labels(d.pop("metadata.labels", UNSET))

        def _parse_metadata_key(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType1
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_key_type_0 = cast(list[str], data)

                return metadata_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_key_type_1 = PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType1.from_dict(data)

                return metadata_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_key_type_2 = PostApiAnalyticsTimeseriesBodyFiltersMetadataKeyType2.from_dict(data)

            return metadata_key_type_2

        metadata_key = _parse_metadata_key(d.pop("metadata.key", UNSET))

        def _parse_metadata_value(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType1
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_value_type_0 = cast(list[str], data)

                return metadata_value_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_value_type_1 = PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType1.from_dict(data)

                return metadata_value_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_value_type_2 = PostApiAnalyticsTimeseriesBodyFiltersMetadataValueType2.from_dict(data)

            return metadata_value_type_2

        metadata_value = _parse_metadata_value(d.pop("metadata.value", UNSET))

        def _parse_metadata_prompt_ids(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType1
            | PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_prompt_ids_type_0 = cast(list[str], data)

                return metadata_prompt_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_prompt_ids_type_1 = PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType1.from_dict(data)

                return metadata_prompt_ids_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_prompt_ids_type_2 = PostApiAnalyticsTimeseriesBodyFiltersMetadataPromptIdsType2.from_dict(data)

            return metadata_prompt_ids_type_2

        metadata_prompt_ids = _parse_metadata_prompt_ids(d.pop("metadata.prompt_ids", UNSET))

        def _parse_traces_error(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType1
            | PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                traces_error_type_0 = cast(list[str], data)

                return traces_error_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                traces_error_type_1 = PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType1.from_dict(data)

                return traces_error_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            traces_error_type_2 = PostApiAnalyticsTimeseriesBodyFiltersTracesErrorType2.from_dict(data)

            return traces_error_type_2

        traces_error = _parse_traces_error(d.pop("traces.error", UNSET))

        def _parse_spans_type(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType1
            | PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                spans_type_type_0 = cast(list[str], data)

                return spans_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                spans_type_type_1 = PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType1.from_dict(data)

                return spans_type_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            spans_type_type_2 = PostApiAnalyticsTimeseriesBodyFiltersSpansTypeType2.from_dict(data)

            return spans_type_type_2

        spans_type = _parse_spans_type(d.pop("spans.type", UNSET))

        def _parse_spans_model(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersSpansModelType1
            | PostApiAnalyticsTimeseriesBodyFiltersSpansModelType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                spans_model_type_0 = cast(list[str], data)

                return spans_model_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                spans_model_type_1 = PostApiAnalyticsTimeseriesBodyFiltersSpansModelType1.from_dict(data)

                return spans_model_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            spans_model_type_2 = PostApiAnalyticsTimeseriesBodyFiltersSpansModelType2.from_dict(data)

            return spans_model_type_2

        spans_model = _parse_spans_model(d.pop("spans.model", UNSET))

        def _parse_evaluations_evaluator_id(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType1
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_evaluator_id_type_0 = cast(list[str], data)

                return evaluations_evaluator_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_evaluator_id_type_1 = (
                    PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType1.from_dict(data)
                )

                return evaluations_evaluator_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_evaluator_id_type_2 = (
                PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdType2.from_dict(data)
            )

            return evaluations_evaluator_id_type_2

        evaluations_evaluator_id = _parse_evaluations_evaluator_id(d.pop("evaluations.evaluator_id", UNSET))

        def _parse_evaluations_evaluator_id_guardrails_only(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_evaluator_id_guardrails_only_type_0 = cast(list[str], data)

                return evaluations_evaluator_id_guardrails_only_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_evaluator_id_guardrails_only_type_1 = (
                    PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1.from_dict(data)
                )

                return evaluations_evaluator_id_guardrails_only_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_evaluator_id_guardrails_only_type_2 = (
                PostApiAnalyticsTimeseriesBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2.from_dict(data)
            )

            return evaluations_evaluator_id_guardrails_only_type_2

        evaluations_evaluator_id_guardrails_only = _parse_evaluations_evaluator_id_guardrails_only(
            d.pop("evaluations.evaluator_id.guardrails_only", UNSET)
        )

        def _parse_evaluations_passed(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType1
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_passed_type_0 = cast(list[str], data)

                return evaluations_passed_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_passed_type_1 = PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType1.from_dict(data)

                return evaluations_passed_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_passed_type_2 = PostApiAnalyticsTimeseriesBodyFiltersEvaluationsPassedType2.from_dict(data)

            return evaluations_passed_type_2

        evaluations_passed = _parse_evaluations_passed(d.pop("evaluations.passed", UNSET))

        def _parse_evaluations_score(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType1
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_score_type_0 = cast(list[str], data)

                return evaluations_score_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_score_type_1 = PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType1.from_dict(data)

                return evaluations_score_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_score_type_2 = PostApiAnalyticsTimeseriesBodyFiltersEvaluationsScoreType2.from_dict(data)

            return evaluations_score_type_2

        evaluations_score = _parse_evaluations_score(d.pop("evaluations.score", UNSET))

        def _parse_evaluations_state(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType1
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_state_type_0 = cast(list[str], data)

                return evaluations_state_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_state_type_1 = PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType1.from_dict(data)

                return evaluations_state_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_state_type_2 = PostApiAnalyticsTimeseriesBodyFiltersEvaluationsStateType2.from_dict(data)

            return evaluations_state_type_2

        evaluations_state = _parse_evaluations_state(d.pop("evaluations.state", UNSET))

        def _parse_evaluations_label(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType1
            | PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_label_type_0 = cast(list[str], data)

                return evaluations_label_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_label_type_1 = PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType1.from_dict(data)

                return evaluations_label_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_label_type_2 = PostApiAnalyticsTimeseriesBodyFiltersEvaluationsLabelType2.from_dict(data)

            return evaluations_label_type_2

        evaluations_label = _parse_evaluations_label(d.pop("evaluations.label", UNSET))

        def _parse_events_event_type(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType1
            | PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_event_type_type_0 = cast(list[str], data)

                return events_event_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_event_type_type_1 = PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType1.from_dict(data)

                return events_event_type_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_event_type_type_2 = PostApiAnalyticsTimeseriesBodyFiltersEventsEventTypeType2.from_dict(data)

            return events_event_type_type_2

        events_event_type = _parse_events_event_type(d.pop("events.event_type", UNSET))

        def _parse_events_metrics_key(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType1
            | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_metrics_key_type_0 = cast(list[str], data)

                return events_metrics_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_metrics_key_type_1 = PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType1.from_dict(data)

                return events_metrics_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_metrics_key_type_2 = PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsKeyType2.from_dict(data)

            return events_metrics_key_type_2

        events_metrics_key = _parse_events_metrics_key(d.pop("events.metrics.key", UNSET))

        def _parse_events_metrics_value(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType1
            | PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_metrics_value_type_0 = cast(list[str], data)

                return events_metrics_value_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_metrics_value_type_1 = PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType1.from_dict(
                    data
                )

                return events_metrics_value_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_metrics_value_type_2 = PostApiAnalyticsTimeseriesBodyFiltersEventsMetricsValueType2.from_dict(data)

            return events_metrics_value_type_2

        events_metrics_value = _parse_events_metrics_value(d.pop("events.metrics.value", UNSET))

        def _parse_events_event_details_key(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType1
            | PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_event_details_key_type_0 = cast(list[str], data)

                return events_event_details_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_event_details_key_type_1 = (
                    PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType1.from_dict(data)
                )

                return events_event_details_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_event_details_key_type_2 = PostApiAnalyticsTimeseriesBodyFiltersEventsEventDetailsKeyType2.from_dict(
                data
            )

            return events_event_details_key_type_2

        events_event_details_key = _parse_events_event_details_key(d.pop("events.event_details.key", UNSET))

        def _parse_annotations_has_annotation(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType1
            | PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                annotations_has_annotation_type_0 = cast(list[str], data)

                return annotations_has_annotation_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                annotations_has_annotation_type_1 = (
                    PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType1.from_dict(data)
                )

                return annotations_has_annotation_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            annotations_has_annotation_type_2 = (
                PostApiAnalyticsTimeseriesBodyFiltersAnnotationsHasAnnotationType2.from_dict(data)
            )

            return annotations_has_annotation_type_2

        annotations_has_annotation = _parse_annotations_has_annotation(d.pop("annotations.hasAnnotation", UNSET))

        def _parse_sentiment_input_sentiment(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType1
            | PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                sentiment_input_sentiment_type_0 = cast(list[str], data)

                return sentiment_input_sentiment_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sentiment_input_sentiment_type_1 = (
                    PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType1.from_dict(data)
                )

                return sentiment_input_sentiment_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            sentiment_input_sentiment_type_2 = (
                PostApiAnalyticsTimeseriesBodyFiltersSentimentInputSentimentType2.from_dict(data)
            )

            return sentiment_input_sentiment_type_2

        sentiment_input_sentiment = _parse_sentiment_input_sentiment(d.pop("sentiment.input_sentiment", UNSET))

        post_api_analytics_timeseries_body_filters = cls(
            topics_topics=topics_topics,
            topics_subtopics=topics_subtopics,
            metadata_user_id=metadata_user_id,
            metadata_thread_id=metadata_thread_id,
            metadata_customer_id=metadata_customer_id,
            metadata_labels=metadata_labels,
            metadata_key=metadata_key,
            metadata_value=metadata_value,
            metadata_prompt_ids=metadata_prompt_ids,
            traces_error=traces_error,
            spans_type=spans_type,
            spans_model=spans_model,
            evaluations_evaluator_id=evaluations_evaluator_id,
            evaluations_evaluator_id_guardrails_only=evaluations_evaluator_id_guardrails_only,
            evaluations_passed=evaluations_passed,
            evaluations_score=evaluations_score,
            evaluations_state=evaluations_state,
            evaluations_label=evaluations_label,
            events_event_type=events_event_type,
            events_metrics_key=events_metrics_key,
            events_metrics_value=events_metrics_value,
            events_event_details_key=events_event_details_key,
            annotations_has_annotation=annotations_has_annotation,
            sentiment_input_sentiment=sentiment_input_sentiment,
        )

        return post_api_analytics_timeseries_body_filters



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_api_traces_by_trace_id_format import GetApiTracesByTraceIdFormat
from ...models.get_api_traces_by_trace_id_llm_mode import GetApiTracesByTraceIdLlmMode
from ...models.get_api_traces_by_trace_id_response_200 import GetApiTracesByTraceIdResponse200
from ...models.get_api_traces_by_trace_id_response_400 import GetApiTracesByTraceIdResponse400
from ...models.get_api_traces_by_trace_id_response_401 import GetApiTracesByTraceIdResponse401
from ...models.get_api_traces_by_trace_id_response_404 import GetApiTracesByTraceIdResponse404
from ...models.get_api_traces_by_trace_id_response_422 import GetApiTracesByTraceIdResponse422
from ...models.get_api_traces_by_trace_id_response_500 import GetApiTracesByTraceIdResponse500
from ...types import UNSET, Response, Unset


def _get_kwargs(
    trace_id: str,
    *,
    format_: GetApiTracesByTraceIdFormat | Unset = UNSET,
    llm_mode: GetApiTracesByTraceIdLlmMode | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_format_: str | Unset = UNSET
    if not isinstance(format_, Unset):
        json_format_ = format_.value

    params["format"] = json_format_

    json_llm_mode: str | Unset = UNSET
    if not isinstance(llm_mode, Unset):
        json_llm_mode = llm_mode.value

    params["llmMode"] = json_llm_mode

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/traces/{trace_id}".format(
            trace_id=quote(str(trace_id), safe=""),
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    GetApiTracesByTraceIdResponse200
    | GetApiTracesByTraceIdResponse400
    | GetApiTracesByTraceIdResponse401
    | GetApiTracesByTraceIdResponse404
    | GetApiTracesByTraceIdResponse422
    | GetApiTracesByTraceIdResponse500
    | None
):
    if response.status_code == 200:
        response_200 = GetApiTracesByTraceIdResponse200.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = GetApiTracesByTraceIdResponse400.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = GetApiTracesByTraceIdResponse401.from_dict(response.json())

        return response_401

    if response.status_code == 404:
        response_404 = GetApiTracesByTraceIdResponse404.from_dict(response.json())

        return response_404

    if response.status_code == 422:
        response_422 = GetApiTracesByTraceIdResponse422.from_dict(response.json())

        return response_422

    if response.status_code == 500:
        response_500 = GetApiTracesByTraceIdResponse500.from_dict(response.json())

        return response_500

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    GetApiTracesByTraceIdResponse200
    | GetApiTracesByTraceIdResponse400
    | GetApiTracesByTraceIdResponse401
    | GetApiTracesByTraceIdResponse404
    | GetApiTracesByTraceIdResponse422
    | GetApiTracesByTraceIdResponse500
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    trace_id: str,
    *,
    client: AuthenticatedClient | Client,
    format_: GetApiTracesByTraceIdFormat | Unset = UNSET,
    llm_mode: GetApiTracesByTraceIdLlmMode | Unset = UNSET,
) -> Response[
    GetApiTracesByTraceIdResponse200
    | GetApiTracesByTraceIdResponse400
    | GetApiTracesByTraceIdResponse401
    | GetApiTracesByTraceIdResponse404
    | GetApiTracesByTraceIdResponse422
    | GetApiTracesByTraceIdResponse500
]:
    """Get a single trace by ID. Defaults to AI-readable digest format.

    Args:
        trace_id (str):
        format_ (GetApiTracesByTraceIdFormat | Unset):
        llm_mode (GetApiTracesByTraceIdLlmMode | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GetApiTracesByTraceIdResponse200 | GetApiTracesByTraceIdResponse400 | GetApiTracesByTraceIdResponse401 | GetApiTracesByTraceIdResponse404 | GetApiTracesByTraceIdResponse422 | GetApiTracesByTraceIdResponse500]
    """

    kwargs = _get_kwargs(
        trace_id=trace_id,
        format_=format_,
        llm_mode=llm_mode,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    trace_id: str,
    *,
    client: AuthenticatedClient | Client,
    format_: GetApiTracesByTraceIdFormat | Unset = UNSET,
    llm_mode: GetApiTracesByTraceIdLlmMode | Unset = UNSET,
) -> (
    GetApiTracesByTraceIdResponse200
    | GetApiTracesByTraceIdResponse400
    | GetApiTracesByTraceIdResponse401
    | GetApiTracesByTraceIdResponse404
    | GetApiTracesByTraceIdResponse422
    | GetApiTracesByTraceIdResponse500
    | None
):
    """Get a single trace by ID. Defaults to AI-readable digest format.

    Args:
        trace_id (str):
        format_ (GetApiTracesByTraceIdFormat | Unset):
        llm_mode (GetApiTracesByTraceIdLlmMode | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GetApiTracesByTraceIdResponse200 | GetApiTracesByTraceIdResponse400 | GetApiTracesByTraceIdResponse401 | GetApiTracesByTraceIdResponse404 | GetApiTracesByTraceIdResponse422 | GetApiTracesByTraceIdResponse500
    """

    return sync_detailed(
        trace_id=trace_id,
        client=client,
        format_=format_,
        llm_mode=llm_mode,
    ).parsed


async def asyncio_detailed(
    trace_id: str,
    *,
    client: AuthenticatedClient | Client,
    format_: GetApiTracesByTraceIdFormat | Unset = UNSET,
    llm_mode: GetApiTracesByTraceIdLlmMode | Unset = UNSET,
) -> Response[
    GetApiTracesByTraceIdResponse200
    | GetApiTracesByTraceIdResponse400
    | GetApiTracesByTraceIdResponse401
    | GetApiTracesByTraceIdResponse404
    | GetApiTracesByTraceIdResponse422
    | GetApiTracesByTraceIdResponse500
]:
    """Get a single trace by ID. Defaults to AI-readable digest format.

    Args:
        trace_id (str):
        format_ (GetApiTracesByTraceIdFormat | Unset):
        llm_mode (GetApiTracesByTraceIdLlmMode | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GetApiTracesByTraceIdResponse200 | GetApiTracesByTraceIdResponse400 | GetApiTracesByTraceIdResponse401 | GetApiTracesByTraceIdResponse404 | GetApiTracesByTraceIdResponse422 | GetApiTracesByTraceIdResponse500]
    """

    kwargs = _get_kwargs(
        trace_id=trace_id,
        format_=format_,
        llm_mode=llm_mode,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    trace_id: str,
    *,
    client: AuthenticatedClient | Client,
    format_: GetApiTracesByTraceIdFormat | Unset = UNSET,
    llm_mode: GetApiTracesByTraceIdLlmMode | Unset = UNSET,
) -> (
    GetApiTracesByTraceIdResponse200
    | GetApiTracesByTraceIdResponse400
    | GetApiTracesByTraceIdResponse401
    | GetApiTracesByTraceIdResponse404
    | GetApiTracesByTraceIdResponse422
    | GetApiTracesByTraceIdResponse500
    | None
):
    """Get a single trace by ID. Defaults to AI-readable digest format.

    Args:
        trace_id (str):
        format_ (GetApiTracesByTraceIdFormat | Unset):
        llm_mode (GetApiTracesByTraceIdLlmMode | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GetApiTracesByTraceIdResponse200 | GetApiTracesByTraceIdResponse400 | GetApiTracesByTraceIdResponse401 | GetApiTracesByTraceIdResponse404 | GetApiTracesByTraceIdResponse422 | GetApiTracesByTraceIdResponse500
    """

    return (
        await asyncio_detailed(
            trace_id=trace_id,
            client=client,
            format_=format_,
            llm_mode=llm_mode,
        )
    ).parsed



"""
Core PromptData structure for prompts.

This module contains the PromptData Pydantic model with conversion methods,
following the TypeScript PromptData interface structure.
"""

from typing import Literal, Optional, List, Union

from pydantic import BaseModel, ConfigDict

from langwatch.generated.langwatch_rest_api_client.types import Unset
from langwatch.generated.langwatch_rest_api_client.models.get_api_prompts_by_id_response_200 import (
    GetApiPromptsByIdResponse200,
)
from langwatch.generated.langwatch_rest_api_client.models.put_api_prompts_by_id_response_200 import (
    PutApiPromptsByIdResponse200,
)
from langwatch.generated.langwatch_rest_api_client.models.post_api_prompts_response_200 import (
    PostApiPromptsResponse200,
)

from .structures import Message, ResponseFormat


class PromptData(BaseModel):
    """
    Core data structure for prompts, matching the TypeScript PromptData interface.

    Contains both core functionality fields and optional metadata for identification/tracing.
    """

    model_config = ConfigDict(extra="ignore")

    # === Core functionality (required) ===
    model: str = ""
    messages: List[Message] = []

    # === Optional core fields ===
    prompt: Optional[str] = None
    temperature: Optional[float] = None
    max_tokens: Optional[int] = None
    response_format: Optional[ResponseFormat] = None

    # === Optional identification (for tracing) ===
    id: Optional[str] = None
    handle: Optional[str] = None
    version: Optional[int] = None
    version_id: Optional[str] = None
    scope: Optional[Literal["PROJECT", "ORGANIZATION"]] = None

    @staticmethod
    def from_api_response(
        response: Union[
            GetApiPromptsByIdResponse200,
            PutApiPromptsByIdResponse200,
            PostApiPromptsResponse200,
        ],
    ) -> "PromptData":
        """
        Create PromptData from API response object.

        Args:
            response: API response object

        Returns:
            PromptData instance with converted fields
        """
        def _unset_to_none(value):
            """Convert UNSET sentinel values to None."""
            return None if isinstance(value, Unset) else value

        messages = []
        raw_messages = _unset_to_none(response.messages)
        if raw_messages:
            messages = [
                Message(role=msg.role.value, content=msg.content)
                for msg in raw_messages
            ]

        # Convert response format if present
        response_format = None
        raw_response_format = _unset_to_none(
            getattr(response, "response_format", None)
        )
        if raw_response_format:
            response_format = ResponseFormat(
                type="json_schema", json_schema=raw_response_format.json_schema
            )

        raw_version = _unset_to_none(response.version)

        return PromptData(
            id=_unset_to_none(response.id),
            handle=_unset_to_none(response.handle),
            model=response.model,
            messages=messages,
            prompt=_unset_to_none(response.prompt),
            temperature=_unset_to_none(response.temperature),
            max_tokens=_unset_to_none(response.max_tokens),
            response_format=response_format,
            version=int(raw_version) if raw_version is not None else None,
            version_id=_unset_to_none(response.version_id),
            scope=response.scope.value if response.scope and not isinstance(response.scope, Unset) else None,
        )



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.post_api_analytics_timeseries_response_200_current_period_item import (
        PostApiAnalyticsTimeseriesResponse200CurrentPeriodItem,
    )
    from ..models.post_api_analytics_timeseries_response_200_previous_period_item import (
        PostApiAnalyticsTimeseriesResponse200PreviousPeriodItem,
    )


T = TypeVar("T", bound="PostApiAnalyticsTimeseriesResponse200")


@_attrs_define
class PostApiAnalyticsTimeseriesResponse200:
    """
    Attributes:
        current_period (list[PostApiAnalyticsTimeseriesResponse200CurrentPeriodItem]):
        previous_period (list[PostApiAnalyticsTimeseriesResponse200PreviousPeriodItem]):
    """

    current_period: list[PostApiAnalyticsTimeseriesResponse200CurrentPeriodItem]
    previous_period: list[PostApiAnalyticsTimeseriesResponse200PreviousPeriodItem]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        current_period = []
        for current_period_item_data in self.current_period:
            current_period_item = current_period_item_data.to_dict()
            current_period.append(current_period_item)

        previous_period = []
        for previous_period_item_data in self.previous_period:
            previous_period_item = previous_period_item_data.to_dict()
            previous_period.append(previous_period_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "currentPeriod": current_period,
                "previousPeriod": previous_period,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_analytics_timeseries_response_200_current_period_item import (
            PostApiAnalyticsTimeseriesResponse200CurrentPeriodItem,
        )
        from ..models.post_api_analytics_timeseries_response_200_previous_period_item import (
            PostApiAnalyticsTimeseriesResponse200PreviousPeriodItem,
        )

        d = dict(src_dict)
        current_period = []
        _current_period = d.pop("currentPeriod")
        for current_period_item_data in _current_period:
            current_period_item = PostApiAnalyticsTimeseriesResponse200CurrentPeriodItem.from_dict(
                current_period_item_data
            )

            current_period.append(current_period_item)

        previous_period = []
        _previous_period = d.pop("previousPeriod")
        for previous_period_item_data in _previous_period:
            previous_period_item = PostApiAnalyticsTimeseriesResponse200PreviousPeriodItem.from_dict(
                previous_period_item_data
            )

            previous_period.append(previous_period_item)

        post_api_analytics_timeseries_response_200 = cls(
            current_period=current_period,
            previous_period=previous_period,
        )

        post_api_analytics_timeseries_response_200.additional_properties = d
        return post_api_analytics_timeseries_response_200

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
